use crate::config::config::STATIC_WS_PWD;
use crate::deport::handle_ws_msg;
use crate::events::{register_client, ClientMessage, Event};
use crate::handles::handshake::handle_handshake;
use crate::handles::online_users_update::send_online_users_and_send;
use dashmap::DashMap;
use futures::{StreamExt};
use lazy_static::lazy_static;
use std::sync::Arc;
use std::time::{Duration, Instant};
use tokio::sync::mpsc::{Receiver, Sender, UnboundedSender};
use tokio::sync::Mutex;
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::Error;

lazy_static! {
    pub static ref ONLINE_USERS: DashMap<String, String> = DashMap::new();
}

pub(crate) async fn handle_client(
    stream: tokio::net::TcpStream,
    event_sender: UnboundedSender<Event>,
    center_to_client_sender: Sender<ClientMessage>,
    mut center_to_client_receiver: Receiver<ClientMessage>,
) -> Result<(), Error> {
    let must_existed_params = ["deviceId", "fromId", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params, STATIC_WS_PWD) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
    .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    let (sender, mut receiver) = ws_stream.split();
    let sender = Arc::new(Mutex::new(sender)); // 包装为 Arc<Mutex<...>>
    let last_heartbeat_time = Arc::new(Mutex::new(Instant::now())); // 包装为 Arc<Mutex<Instant>>

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromId") {
            let from_id = from_id.clone();
            let from_id_clone = from_id.clone();

            // 注册客户端到事件中心
            register_client(from_id.clone(), center_to_client_sender).await;

            // 发送新连接事件
            event_sender
                .send(Event::NewConnection(from_id.clone(), params))
                .unwrap();

            println!("断开与用户id: {}，连接", from_id_clone);
            loop {
                let sender_clone = Arc::clone(&sender);
                let from_id_clone = from_id.clone();
                let event_sender_clone = event_sender.clone();
                let last_heartbeat_time_clone = Arc::clone(&last_heartbeat_time);
                tokio::select! {
                    // 处理消息接收
                    maybe_msg = receiver.next() => {
                        if let Some(msg) = maybe_msg {
                            tokio::spawn(handle_ws_msg(
                                msg?,
                                sender_clone,
                                from_id_clone,
                                event_sender_clone,
                                last_heartbeat_time_clone,
                            ));
                        }
                    }
                    // 定期检查事件中心的消息
                    msg = center_to_client_receiver.recv() => {
                        if let Some(msg) = msg {
                            println!("消息中心：==> 收到消息 {:?}", &msg);
                            match msg {
                                ClientMessage::CmdUpdateOnlineUsers => {
                                    println!("消息中心：==> 收到 CmdUpdateOnlineUsers 消息 发送给 {}", &from_id_clone);
                                    if let Err(e) = send_online_users_and_send(&mut *sender_clone.lock().await, &from_id_clone).await {
                                        println!("处理在线用户列表出错了：{:?}", e);
                                        // 发送关闭连接事件
                                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    // 处理心跳超时
                    _ = time::sleep_until(tokio::time::Instant::from(*last_heartbeat_time.lock().await) + Duration::from_secs(20)) => {
                        println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id_clone);
                        // 发送关闭连接事件
                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                        break;
                    }
                }
            }
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}
