use std::sync::{Arc};
use std::time::Instant;
use futures::SinkExt;
use crate::events::Event;
use futures::stream::SplitSink;
use tokio::net::TcpStream;
use tokio::sync::mpsc::UnboundedSender;
use tokio::sync::Mutex;
use tokio_tungstenite::WebSocketStream;
use tungstenite::Message;
use crate::handles::handle_messages::handle_other_message;
use crate::handles::heartbeat::{heart_resp};
use crate::handles::online_users_update::send_online_users_and_send;
use crate::utils::json_utils::parse_message;

pub async fn handle_ws_msg(
    msg: Message,
    sender: Arc<Mutex<SplitSink<WebSocketStream<TcpStream>, Message>>>,
    from_id: String,
    event_sender: UnboundedSender<Event>,
    last_heartbeat_time: Arc<Mutex<Instant>>,
) {
    if msg.is_text() {
        let text = msg.to_text().unwrap();
        match parse_message(text) {
            Ok(data) => {
                match data.msg_type.as_str() {
                    "Heart" => {
                        println!("收到客户端心跳消息 {:?}", &data);
                        *last_heartbeat_time.lock().await = Instant::now();
                        if let Ok(json_str) = heart_resp(&from_id) {
                            let mut sender = sender.lock().await;
                            if let Err(e) = sender.send(Message::text(json_str)).await {
                                println!("发送心跳信息失败: {}", e);
                                // 发送关闭连接事件
                                event_sender.send(Event::CloseConnection(from_id.clone())).unwrap();
                            }
                        }
                    },
                    "GetOnlineUserList" => {
                        println!("收到客户端获取在线用户列表 {:?}", &data);
                        let mut sender = sender.lock().await;
                        if let Err(e) = send_online_users_and_send(&mut *sender, &from_id).await {
                            println!("处理在线用户列表出错了：{:?}", e);
                            // 发送关闭连接事件
                            event_sender.send(Event::CloseConnection(from_id.clone())).unwrap();
                        }
                    },
                    _ => {
                        let mut sender = sender.lock().await;
                        if let Err(e) = handle_other_message(&mut *sender, &data, &from_id).await {
                            println!("Failed to handle other message: {}", e);
                            // 发送关闭连接事件
                            event_sender.send(Event::CloseConnection(from_id.clone())).unwrap();
                        }
                    }
                }
            }
            Err(e) => {
                println!("解析JSON数据出错: {}", e);
            }
        }
    }
}